"""
Simple interactive process used to demonstrate the use of the
PyInteractiveConsole widget.
"""
print('running an interactive process that will raise an error soon')

# traceback will be highlighted
raise RuntimeError()
